use rand::random;

fn approximate_x(random_number: f32) -> f32 {
    let mut x: f32 = 0f32;
    while random_number
        > (2f32 / std::f32::consts::PI).mul_add(
            (x - 1f32).mul_add(
                (x - 1f32).mul_add(-(x - 1f32), 1f32).sqrt(),
                (x - 1f32).asin(),
            ),
            1f32,
        )
    {
        x += 0.000_1;
    }
    x
}

fn calculate_y(random_number: f32, x: &f32) -> f32 {
    random_number * 2f32 * (x - 1f32).mul_add(-(x - 1f32), 1f32).sqrt()
        - (x - 1f32).mul_add(-(x - 1f32), 1f32).sqrt()
        + 1f32
}

fn mean(distribution: &[f32]) -> f32 {
    distribution.iter().sum::<f32>() / SAMPLE_SIZE as f32
}

fn dispersion(mean: f32, distribution: &[f32]) -> f32 {
    distribution
        .iter()
        .map(|x| (*x - mean).powi(2))
        .sum::<f32>()
        / (SAMPLE_SIZE - 1) as f32
}

fn standard_deviation(mean: f32, distribution: &[f32]) -> f32 {
    dispersion(mean, distribution).sqrt()
}

fn covariance(distribution_x: &[f32], mean_x: f32, distribution_y: &[f32], mean_y: f32) -> f32 {
    let mut sum: f32 = 0f32;
    for i in 0..SAMPLE_SIZE {
        sum += (distribution_x[i] - mean_x) * (distribution_y[i] - mean_y);
    }
    sum / (SAMPLE_SIZE - 1) as f32
}

const SAMPLE_SIZE: usize = 5_000;

fn main() {
    let x: Vec<f32> = (0..SAMPLE_SIZE)
        .map(|_| approximate_x(random::<f32>()))
        .collect();
    let mut y: Vec<f32> = Vec::new();

    x.iter()
        .for_each(|x| y.push(calculate_y(random::<f32>(), x)));

    let mean_x = mean(&x);
    let mean_y = mean(&y);

    println!("Середнє значення розподілу x: {mean_x}");
    println!("Середнє значення розподілу y: {mean_y}");

    let x_deviation = standard_deviation(mean_x, &x);
    let y_deviation = standard_deviation(mean_y, &y);

    println!("Середньоквадратичне відхилення розподілу x: {x_deviation}");
    println!("Середньоквадратичне відхилення розподілу y: {y_deviation}");

    let covariance = covariance(&x, mean_x, &y, mean_y);
    let correlation = covariance / (x_deviation * y_deviation);

    println!("Коефіцієнт кореляції: {correlation}");
}
