use rand::{thread_rng, Rng};

const SAMPLE_SIZE: usize = 1000;

#[derive(Debug)]
enum State {
    One,
    Two,
    Three,
    Four,
    Five,
}

#[derive(Debug)]
struct TotalStats {
    state: State,
    state_counter: Vec<u32>,
}

impl TotalStats {
    fn new() -> Self {
        Self {
            state: State::One,
            state_counter: vec![0; 5],
        }
    }
    fn change_state(&mut self, random_number: f32) {
        match &self.state {
            State::One => {
                if (0f32..0.2).contains(&random_number) {
                    self.state = State::One;
                    self.state_counter[0] += 1;
                } else if (0.2..0.7).contains(&random_number) {
                    self.state = State::Two;
                    self.state_counter[1] += 1;
                } else if (0.7..1f32).contains(&random_number) {
                    self.state = State::Five;
                    self.state_counter[4] += 1;
                }
            }
            State::Two => {
                if (0f32..0.35).contains(&random_number) {
                    self.state = State::Three;
                    self.state_counter[2] += 1;
                } else if (0.35..1f32).contains(&random_number) {
                    self.state = State::Four;
                    self.state_counter[3] += 1;
                }
            }
            State::Three => {
                if (0f32..0.2).contains(&random_number) {
                    self.state = State::One;
                    self.state_counter[0] += 1;
                } else if (0.2..1f32).contains(&random_number) {
                    self.state = State::Four;
                    self.state_counter[3] += 1;
                }
            }
            State::Four => {
                if (0f32..1f32).contains(&random_number) {
                    self.state = State::Five;
                    self.state_counter[4] += 1;
                }
            }
            State::Five => {
                if (0f32..0.45).contains(&random_number) {
                    self.state = State::One;
                    self.state_counter[0] += 1;
                } else if (0.45..1f32).contains(&random_number) {
                    self.state = State::Four;
                    self.state_counter[3] += 1;
                }
            }
        }
    }

    fn probabilities(&self) -> Vec<f32> {
        let mut probabilities: Vec<f32> = Vec::new();

        self.state_counter
            .iter()
            .for_each(|i| probabilities.push(*i as f32 / SAMPLE_SIZE as f32));

        probabilities
    }
}

fn main() {
    let mut rng = thread_rng();

    let mut stats = TotalStats::new();

    (0..SAMPLE_SIZE).for_each(|_| stats.change_state(rng.gen()));

    stats
        .probabilities()
        .iter()
        .enumerate()
        .for_each(|(i, p)| println!("p{i} = {p}"));
}
